@@ -5,26 +5,17 @@ our $AUTHORITY = 'cpan:GENE';
 use strict;
 use warnings;
 
-our $VERSION = '0.0505';
+our $VERSION = '0.06';
 
 use Carp;
 use Game::Battleship::Player;
+use Moo;
 
 
-sub new {
-    my $class = shift;
-    my $self = {};
-    bless $self, $class;
-    $self->_setup(@_);
-    return $self;
-}
-
-sub _setup {
-    my ($self, @players) = @_;
-    # Set up a default, two player game if no players are given.
-    @players = ('', '') unless @players;
-    $self->add_player($_) for @players;
-}
+has players => (
+    is  => 'rw',
+    isa => sub { croak 'Invalid players list' unless ref($_[0]) eq 'HASH' },
+);
 
 
 sub add_player {
@@ -35,7 +26,7 @@ sub add_player {
         # ..find the least whole number that is not already used.
         my @nums = sort { $a <=> $b }
             grep { s/^player_(\d+)$/$1/ }
-                keys %$self;
+                keys %{ $self->{players} };
         my $n = 1;
         for (@nums) {
             last if $n > $_;
@@ -48,7 +39,7 @@ sub add_player {
     my $who = "player_$i";
 
     # Return undef if we are trying to add an existing player.
-    if ( exists $self->{$who} ) {
+    if ( exists $self->{players}{$who} ) {
         warn "A player number $i already exists\n";
         return;
     }
@@ -58,11 +49,11 @@ sub add_player {
 
     # We are given a player object.
     if (ref ($player) eq 'Game::Battleship::Player') {
-        $self->{$who} = $player;
+        $self->{players}{$who} = $player;
     }
     # We are given the guts of a player.
     elsif (ref ($player) eq 'HASH') {
-        $self->{$who} = Game::Battleship::Player->new(
+        $self->{players}{$who} = Game::Battleship::Player->new(
             id => $player->{id} || $i,
             name => $player->{name} || $who,
             fleet => $player->{fleet},
@@ -71,17 +62,14 @@ sub add_player {
     }
     # We are just given a name.
     else {
-        $self->{$who} = Game::Battleship::Player->new(
+        $self->{players}{$who} = Game::Battleship::Player->new(
             id   => $i,
             name => $player,
         );
     }
 
-    # Add the player object reference to the list of game players.
-    push @{ $self->{players} }, $self->{$who};
-
     # Hand the player object back.
-    return $self->{$who};
+    return $self->{players}{$who};
 }
 
 
@@ -90,16 +78,14 @@ sub player {
     my $player;
 
     # Step through each player...
-    for (keys %$self) {
-        next unless /^player_/;
-
+    for (keys %{ $self->{players} }) {
         # Are we looking at the same player name, key or number?
         if( $_ eq $name ||
-            $self->{$_}{name} eq $name ||
-            $self->{$_}{id} eq $name
+            $self->{players}{$_}{name} eq $name ||
+            $self->{players}{$_}{id} eq $name
         ) {
             # Set the player object to return.
-            $player = $self->{$_};
+            $player = $self->{players}{$_};
             last;
         }
     }
@@ -109,20 +95,17 @@ sub player {
 }
 
 
-sub players { return shift->{players} }
-
-
 sub play {
     my ($self, %args) = @_;
     my $winner = 0;
 
     while (not $winner) {
         # Take a turn per live player.
-        for my $player (@{ $self->{players} }) {
+        for my $player (values %{ $self->{players} }) {
             next unless $player->{life};
 
             # Strike each opponent.
-            for my $opponent (@{ $self->{players} }) {
+            for my $opponent (values %{ $self->{players} }) {
                 next if $opponent->{name} eq $player->{name} ||
                     !$opponent->{life};
 
@@ -137,11 +120,11 @@ sub play {
         }
 
         # Do we have a winner?
-        my @alive = grep { $_->{life} } @{ $self->{players} };
+        my @alive = grep { $self->{players}{$_}{life} } keys %{ $self->{players} };
         $winner = @alive == 1 ? shift @alive : undef;
     }
 
-warn $winner->name ." is the winner!\n";
+#warn $winner->name ." is the winner!\n";
     return $winner;
 }
 
@@ -175,12 +158,13 @@ Game::Battleship - "You sunk my battleship!"
 
 =head1 VERSION
 
-version 0.0505
+version 0.06
 
 =head1 SYNOPSIS
 
   use Game::Battleship;
-  my $g = Game::Battleship->new(qw( Gene Aaron ));
+  my $g = Game::Battleship->new;
+  $g->add_player('Aaron');
   $g->add_player('Tabi');
   my $winner = $g->play();
   print $winner->name(), " wins!\n";
@@ -209,26 +193,10 @@ Game::Battleship - "You sunk my battleship!"
 =head2 B<new()>
 
   $g = Game::Battleship->new;
-  $g = Game::Battleship->new($name);
-  $g = Game::Battleship->new($player_object);
-  $g = Game::Battleship->new(
-      { name => $name,
-        fleet => \@fleet,
-        dimensions => [ $width, $height ] },
-  );
+  $g = Game::Battleship->new( players => [$player1, $player2] );
 
 Construct a new C<Game::Battleship> object.
 
-The players can be given as strings, C<Game::Battleship::Player>
-objects or as hash references containing player attributes.
-
-If not given explicitly, 'player_1' and 'player_2' are used as the
-player names with two 10 x 10 grids and 5 predetermined ships are setup
-for each.
-
-See L<Game::Battleship::Player> for details on the default and custom
-settings.
-
 =head2 B<add_player()>
 
   $g->add_player;
@@ -253,6 +221,9 @@ If this number is not provided, the least whole number that is not
 represented in the player IDs is used.  If a player already exists
 with that number, a warning is emitted and the player is not added.
 
+See L<Game::Battleship::Player> for details on the default and custom
+settings.
+
 =head2 B<player()>
 
   $player_obj = $g->player($name);
@@ -263,8 +234,6 @@ Return the C<Game::Battle::Player> object that matches the given
 name, key or number (where the key is C</player_\d+/> and the number
 is just the numeric part of the key).
 
-=head2 B<players()>
-
 =head2 B<play()>
 
   $winner = $g->play;
