@@ -1,27 +1,28 @@
 package Game::Battleship::Grid;
-$Game::Battleship::Grid::VERSION = '0.0505';
+$Game::Battleship::Grid::VERSION = '0.06';
 our $AUTHORITY = 'cpan:GENE';
-$VERSION = '0.0302';
 use strict;
 use warnings;
 use Carp;
 use Game::Battleship::Craft;
 
-sub new {
-    my ($proto, %args) = @_;
-    my $class = ref ($proto) || $proto;
-    my $self = {
-        # User defined or use the standard, zero based one.
-        dimension => $args{dimension} || [9, 9],
-    };
-    bless $self, $class;
-    $self->_init($args{fleet});
-    return $self;
-}
+use Moo;
+use Types::Standard qw( ArrayRef Int );
+
+has dimension => (
+    is      => 'ro',
+    isa     => ArrayRef[Int],
+    default => sub { [ 9, 9 ] },
+);
+
+has fleet => (
+    is  => 'ro',
+    isa => ArrayRef,
+);
 
 # Place the array reference of craft on the grid.
-sub _init {
-    my ($self, $fleet) = @_;
+sub BUILD {
+    my $self = shift;
 
     # Initialize the matrix.
     for my $i (0 .. $self->{dimension}[0]) {
@@ -31,7 +32,7 @@ sub _init {
     }
 
     # Place the fleet on the grid.
-    for my $craft (@$fleet) {
+    for my $craft (@{ $self->{fleet} }) {
         my ($ok, $x0, $y0, $x1, $y1, $orient);
 
         if (defined $craft->{position}) {
@@ -67,7 +68,7 @@ sub _init {
                     # a position, do the craft share a common point?
                     my $collide = 0;
 
-                    for (@$fleet) {
+                    for (@{ $self->{fleet} }) {
                         # Ships can't be the same.
                         if ($craft->{name} ne $_->{name}) {
                             # Ships can't intersect.
@@ -225,7 +226,7 @@ Game::Battleship::Grid
 
 =head1 VERSION
 
-version 0.0505
+version 0.06
 
 =head1 SYNOPSIS
 
@@ -262,7 +263,7 @@ non-overlapping positions.
 Naturally, it is required that the combined sizes of the ships be
 less than the area of the grid.
 
-=item * dimensions => [$WIDTH, $HEIGHT]
+=item * dimension => [$WIDTH, $HEIGHT]
 
 Optional array reference with the grid height and width values.
 
@@ -270,6 +271,10 @@ If not provided, the standard ten by ten playing surface is used.
 
 =back
 
+=head2 B<BUILD>
+
+Setup
+
 =head1 PRIVATE FUNCTIONS
 
 =over 4
