@@ -1,23 +1,35 @@
 package Game::Battleship::Player;
-$Game::Battleship::Player::VERSION = '0.0505';
+$Game::Battleship::Player::VERSION = '0.06';
 our $AUTHORITY = 'cpan:GENE';
-$VERSION = '0.0502';
 use strict;
 use warnings;
 use Carp;
 use Game::Battleship::Craft;
 use Game::Battleship::Grid;
 
-sub new {
-    my ($proto, %args) = @_;
-    my $class = ref ($proto) || $proto;
+use Moo;
+use Types::Standard qw( ArrayRef Int Str );
 
-    my $self = {
-        id    => $args{id}    || undef,
-        name  => $args{name}  || undef,
-        score => $args{score} || 0,
-        life  => $args{life}  || 0,
-        fleet => $args{fleet} || [
+has id => (
+    is  => 'ro',
+    isa => Str,
+);
+
+has name => (
+    is  => 'ro',
+    isa => Str,
+);
+
+has life => (
+    is  => 'ro',
+    isa => Int,
+);
+
+has fleet => (
+    is      => 'ro',
+    isa     => ArrayRef,
+    default => sub {
+        [
             Game::Battleship::Craft->new(
                 name   => 'aircraft carrier',
                 points => 5,
@@ -38,32 +50,35 @@ sub new {
                 name   => 'destroyer',
                 points => 2,
             ),
-        ],
-    };
+        ]
+    },
+);
 
-    bless $self, $class;
-    $self->_init(\%args);
-    return $self;
-}
+has dimensions => (
+    is  => 'ro',
+    isa => ArrayRef[Int],
+);
 
-sub _init {
-    my ($self, $args) = @_;
+has grid => (
+    is  => 'ro',
+    isa => ArrayRef,
+);
 
-    # Initialize a grid and place the player's fleet, if one is
-    # provided.
-    $self->{grid} = Game::Battleship::Grid->new(
-        fleet => $self->{fleet},
-        dimensions => $args->{dimensions},
-    );
+sub BUILD {
+    my $self = shift;
+
+    $self->{grid} =
+        Game::Battleship::Grid->new(
+            fleet      => $self->{fleet},
+            dimensions => $self->{dimensions},
+        );
 
     # Compute the life points for this player.
     $self->{life} += $_->{points} for @{ $self->{fleet} };
 }
 
-sub name { return shift->{name} }
-
 # The enemy must be a G::B::Player object.
-sub grid {
+sub matrix {
     my ($self, $enemy) = @_;
     return $enemy
         ? join "\n",
@@ -175,7 +190,7 @@ Game::Battleship::Player
 
 =head1 VERSION
 
-version 0.0505
+version 0.06
 
 =head1 SYNOPSIS
 
@@ -216,6 +231,22 @@ Game::Battleship::Player - A Battleship player class
 
 =over 4
 
+=item * id => $STRING
+
+Computed
+
+=item * grid => $STRING
+
+Computed
+
+=item * life => $STRING
+
+Computed
+
+=item * id => $STRING
+
+Computed
+
 =item * name => $STRING
 
 An optional attribute provided to give the player a name.
@@ -238,10 +269,14 @@ ten by ten grid is used.
 
 =back
 
-=item B<grid>
+=item B<BUILD>
 
-  $grid = $player->grid();
-  $grid = $player->grid($enemy);
+Setup
+
+=item B<matrix>
+
+  $grid = $player->matrix();
+  $grid = $player->matrix($enemy);
 
 Return the playing grid as a "flush-left" text matrix like this:
 
@@ -256,9 +291,6 @@ Return the playing grid as a "flush-left" text matrix like this:
   . . . . . A . . . B
   . . . . . A . . . B
 
-Eventually, this method will respect the game type and return an
-appropriate representation, such as a PNG file or XML, etc.
-
 =item B<strike> $PLAYER, @COORDINATE
 
   $strike = $player->strike($enemy, $x, $y);
